import cndprint
from ._inventory import Inventory  # noqa: F401
from ._workflow import Workflow  # noqa: F401
from ._team import Team  # noqa: F401
from ._organization import Organization  # noqa: F401
from ._host import Host  # noqa: F401
from ._group import Group  # noqa: F401
from ._base import Base
import os


class CndAap(Base):
    def __init__(self, host, creds={"username": None, "password": None}, _print=cndprint.CndPrint()):
        super().__init__(host, creds=creds, _print=_print)
        self._get_all_subclass()
        for sub_class_name in CndAap._get_all_subclass():
            sub_class = eval(sub_class_name)
            setattr(self, sub_class.__name__.lower(), sub_class(host, creds, self._print))

    @staticmethod
    def _filename_to_camel_case(filename):
        str = filename.split('.')[0][0:]
        temp = str.split('_')
        res = ''.join(ele.title() for ele in temp[0:])
        return res

    @staticmethod
    def _get_all_subclass():
        files = filter(lambda my_file: (my_file[0] == '_' and my_file[1] != '_' and my_file != '_base.py'), os.listdir("src/aap"))
        my_list = list(files)
        return [CndAap._filename_to_camel_case(my_file) for my_file in my_list]



import sys
from mockito import when, mock, unstub
from expects import *
from mamba import description, context, it
from requests_mock import Mocker
import yaml
import src.aap as aap
import tests.vars as vars
import cndprint
import time


cnd_print = cndprint.CndPrint(level="Info", uuid="MambaTest", silent_mode=False)
host = ''
with description('CndApp') as self:
    with before.each:
        unstub()
        self.instance = aap.cnd_aap.CndAap(vars.host, _print=vars._print)
        self.instance = self.instance.workflow

    with context("__init__"):
        with it("shoud get an instance"):
            expect(isinstance(self.instance, aap._workflow.Workflow)).to(equal(True))

    with context("data"):
        with it("shoud support the right field"):
            result = self.instance.data("name", "inventory_id", description="description", variables={"a": "b"})
            expected_result = {
                'name': 'name',
                'description': 'description',
                'inventory': 'inventory_id',
                'variables': {'a': 'b'}
            }
            expect(result).to(equal(expected_result))

    with context("_execute", "_execute"):
        with it("shoud return id of the workflow_job"):
            when(self.instance).find_id(...).thenReturn(123)
            with Mocker(real_http=True) as m:
                json_data = vars.read_yaml_file("tests/data/workflows/execute.json")
                m.post(f'{vars.host}/api/v2/workflow_job_templates/123/launch/', status_code=200, json=json_data)
                result = self.instance._execute("My-Workflow", 99)
                expect(result).to(equal(234))

    with context("_wait", "_wait"):
        with before.each:
            when(time).sleep(...).thenReturn(None)
                
        with it("shoud False on failed"):
            when(self.instance).find_id(...).thenReturn(123)
            json_data_wait = vars.read_yaml_file("tests/data/workflows/wait.json")
            json_data_wait_failed = vars.read_yaml_file("tests/data/workflows/wait-failed.json")
            when(self.instance)._query(...).thenReturn({"content": json_data_wait}).thenReturn({"content": json_data_wait_failed})
            result = self.instance._wait(99)
            expect(result).to(equal(False))
                
        with it("shoud True on success"):
            when(self.instance).find_id(...).thenReturn(123)
            json_data_wait = vars.read_yaml_file("tests/data/workflows/wait.json")
            json_data_wait_failed = vars.read_yaml_file("tests/data/workflows/wait-success.json")
            when(self.instance)._query(...).thenReturn({"content": json_data_wait}).thenReturn({"content": json_data_wait_failed})
            result = self.instance._wait(99)
            expect(result).to(equal(True))
                
        with it("shoud None if time is over"):
            when(self.instance).find_id(...).thenReturn(123)
            json_data_wait = vars.read_yaml_file("tests/data/workflows/wait.json")
            json_data_wait_failed = vars.read_yaml_file("tests/data/workflows/wait-success.json")
            when(self.instance)._query(...).thenReturn({"content": json_data_wait})
            result = self.instance._wait(99)
            expect(result).to(equal(None))

    with context("execute_and_wait", "execute_and_wait"):
        with it("shoud execute and wait and return True if success"):
            when(self.instance)._execute(...).thenReturn(None)
            when(self.instance)._wait(...).thenReturn(True)
            result = self.instance.execute_and_wait("name", "inventory_id")
            expect(result).to(equal(True))
            
        with it("shoud execute and wait and return False if failed"):
            when(self.instance)._execute(...).thenReturn(None)
            when(self.instance)._wait(...).thenReturn(False)
            result = self.instance.execute_and_wait("name", "inventory_id")
            expect(result).to(equal(False))




import cndprint
import yaml


creds = {"username": '', "password": ''}
host = 'https://iamnotexisting.com'
_print = cndprint.CndPrint(level="log", uuid=">> ", silent_mode=False)

def read_file(filename):
    return open(filename).read()
    
def read_yaml_file(filename):
    content = read_file(filename)
    return yaml.safe_load(content)



import requests
import cndprint
import urllib3
import json
urllib3.disable_warnings()


class Base:
    prefix = "/api/v2/"
    home_path = ''

    def __init__(self, host, creds={"username": None, "password": None}, _print=cndprint.CndPrint()):
        self._host = host
        self._creds = creds
        self._print = _print
        self.default_headers = {"Content-Type": "application/json"}

    def data(self, name):
        return {"name": name}

    def api(self):
        result = self._query('me')
        return result["content"]

    def display_log(self, url, method, response):
        self._print.trace_v(f'Url: {url}')
        self._print.trace_v(f'Method: {method}')
        self._print.trace_v(f'Status Code: {response.status_code}')
        self._print.trace_v(f'Response Content: {response.content}')

    def _query(self, url, method="get", json_data=None, retry=False, retry_count=3):
        if method not in ['get', 'post', 'delete', 'patch', 'put']:
            raise AttributeError("(CND) Method not allowed")
        url = url.replace(self.prefix, '')
        full_url = f"{self._host}{self.prefix}{url}"
        self.kwargs = {}
        if json_data is not None:
            self.kwargs["json"] = json_data
        response = getattr(requests, method)(
            full_url,
            headers=self.default_headers,
            auth=(self._creds["username"], self._creds["password"]),
            verify=False,
            **self.kwargs
        )
        self.display_log(full_url, method, response)
        if response.status_code in [200, 201, 202, 204, 404]:
            return {
                "content": json.loads(response.content),
                "status_code": response.status_code
            }
        self._print.info_e(f'Error with status code: {response.status_code}')
        return False

    def find_all(self, next_page=None):
        self._print.log_d(f"find_all {type(self).__name__}")
        url = next_page if next_page is not None else self.home_path
        result = self._query(url)
        items = result["content"]["results"]
        if result["content"]["next"] is not None:
            items += self.find_all(result["content"]["next"])
        return items

    def destroy(self, id):
        self._print.log_d(f"Find_id {type(self).__name__}")
        result = self._query(f'{self.home_path}/{id}', method="delete")
        if result["status_code"] == 204:
            return True
        return False

    def find_id(self, name):
        self._print.log_d(f"Find_id {type(self).__name__}: {name}")
        result = self._query(f'{self.home_path}?search={name}')
        return result["content"]["results"][0]["id"]

    def _create(self, data):
        self._print.log_d(f"Create {type(self).__name__}")
        result = self._query(f"{self.home_path}", method="post", json_data=data)
        return result["content"]["id"]

    def _update(self, id, data):
        self._print.log_d(f"Update {type(self).__name__}")
        result = self._query(f"{self.home_path}", method="put", json_data=data)
        return result["content"]["id"]

    def create_or_update(self, data):
        self._print.log_d(f"create_or_update {type(self).__name__}")
        id = self.find_id(data["name"])
        if id is not None:
            return self._update(id, data)
        return self._create(data)



from ._base import Base


class Team(Base):
    home_path = 'teams'

    def data(self, name, organization_id, description=None):
        description = description if description is not None else ''
        return {"name": name, "description": description, "organization": organization_id}



from ._base import Base


class Inventory(Base):
    home_path = 'inventories'

    def data(self, name, organization_id, description=None, variables={}):
        description = description if description is not None else ''
        return {"name": name, "description": description, "organization": organization_id, "variables": variables, "kind": ""}



from mockito import when, mock, unstub
from expects import *
from mamba import description, context, it
from requests_mock import Mocker
import tests.vars as vars
import src.aap as aap


with description('Organization') as self:
    with before.each:
        unstub()
        self.instance = aap.cnd_aap.CndAap(vars.host, _print=vars._print)
        self.instance = self.instance.organization

    with context("__init__"):
        with it("shoud get an instance"):
            expect(isinstance(self.instance, aap._organization.Organization)).to(equal(True))

    with context("data", "data"):
        with it("shoud return data"):
            result = self.instance.data('Abc')
            expect(result).to(equal({"name": "Abc", "description": ''}))

    with context("_update", "_update"):
        with it("shoud return organization id"):
            with Mocker(real_http=True) as m:
                result = vars.read_yaml_file("tests/data/organizations/create.json")
                m.put(f'{vars.host}/api/v2/organizations', status_code=201, json=result)
                result = self.instance._update(id, self.instance.data('Abc'))
                expect(result).to(equal(456))

    with context("_create", "_create"):
        with it("shoud return organization id"):
            with Mocker(real_http=True) as m:
                result = vars.read_yaml_file("tests/data/organizations/create.json")
                m.post(f'{vars.host}/api/v2/organizations', status_code=201, json=result)
                result = self.instance._create(self.instance.data('Abc'))
                expect(result).to(equal(456))

    with context("find_id", "find_id"):
        with it("shoud return organization id"):
            with Mocker(real_http=True) as m:
                result = vars.read_yaml_file("tests/data/organizations/find.json")
                m.get(f'{vars.host}/api/v2/organizations?search=Org_name', status_code=200, json=result)
                result = self.instance.find_id("Org_name")
                expect(result).to(equal(123))

    with context("find_all", "find_all"):
        with it("shoud return multiple"):
            with Mocker(real_http=True) as m:
                page1 = vars.read_yaml_file("tests/data/organizations/find_all-page1.json")
                page2 = vars.read_yaml_file("tests/data/organizations/find_all-page2.json")
                m.get(f'{vars.host}/api/v2/organizations', status_code=200, json=page1)
                m.get(f'{vars.host}/api/v2/organizations?page=2', status_code=200, json=page2)
                result = self.instance.find_all()
                expect(len(result)).to(equal(3))



import sys
from mockito import when, mock, unstub
from expects import *
from mamba import description, context, it
import yaml
import src.aap as aap
import tests.vars as vars


with description('CndApp') as self:
    with before.each:
        unstub()
        self.instance = aap.cnd_aap.CndAap(vars.host)

    with context("__init__"):
        with it("shoud set the host, username and password"):
            expect(self.instance._host).to(equal(vars.host))
